/*
    (C) 1995-97 AROS - The Amiga Research OS
    $Id: getprogramname.c,v 1.1.1.1 2005/03/15 15:54:40 laire Exp $

    Desc: Get the name of the current program.
    Lang: english
*/
#include <proto/exec.h>
#include <dos/dos.h>
#include "dos_intern.h"

/*****************************************************************************

    NAME */
#include <proto/dos.h>

	AROS_LH2(LONG, GetProgramName,

/*  SYNOPSIS */
	AROS_LHA(STRPTR, buf, D1),
	AROS_LHA(LONG  , len, D2),

/*  LOCATION */
	struct DosLibrary *, DOSBase, 96, Dos)

/*  FUNCTION
	Copies the name of the current program from the CLI structure
	into the buffer. If the buffer is too small the name is truncated,
	and a failure is returned. If the current process doesn't have
	a CLI structure, a 0 length string is put into the buffer and a
	failure is returned.

    INPUTS
	buf - Buffer for the name.
	len - Size of the buffer in bytes.

    RESULT
	!=0 on success, 0 on failure. IoErr() gives additional information
	in that case.

    NOTES

    EXAMPLE

    BUGS

    SEE ALSO
	SetProgramName()

    INTERNALS

    HISTORY
	27-11-96    digulla automatically created from
			    dos_lib.fd and clib/dos_protos.h

*****************************************************************************/
{
    AROS_LIBFUNC_INIT
    AROS_LIBBASE_EXT_DECL(struct DosLibrary *,DOSBase)

    struct Process *me = (struct Process *)FindTask(NULL);
    struct CommandLineInterface *cli = me->pr_Task.tc_Node.ln_Type == NT_PROCESS ? BADDR(me->pr_CLI) : NULL;
    STRPTR cname;
    ULONG clen;
    LONG ret = DOSTRUE;

    DEBUG_ACCESSOR(dprintf("GetProgramName: me = $%lx, cli = $%lx, buf = $%lx, len = %ld\n",
			   (ULONG) me, (ULONG) cli, (ULONG) buf, len));

    FAILFORTASK(me, return DOSFALSE;)

    if (cli == NULL)
    {
	*buf = '\0';
	me->pr_Result2 = ERROR_OBJECT_WRONG_TYPE;
	return DOSFALSE;
    }

    cname = AROS_BSTR_ADDR(cli->cli_CommandName);
    clen = (ULONG)AROS_BSTR_strlen(cli->cli_CommandName);
    if (clen >= len)
    {
	clen = len - 1;
	me->pr_Result2 = ERROR_LINE_TOO_LONG;
	ret = DOSFALSE;
    }
    else
    {
	/* 13-Jun-2003 bugfix: Explicitly set ioerr to 0 for success. - Piru */
	me->pr_Result2 = 0;
    }

    CopyMem(cname, buf, clen);
    buf[clen] = '\0';

    DEBUG_ACCESSOR(dprintf("GetProgramName: name = \"%s\", ret = %ld\n", buf, (LONG) ret));

    return ret;
    AROS_LIBFUNC_EXIT
} /* GetProgramName */
